#!/bin/sh
# $Id: t-dkim-0.sh,v 1.16 2013/12/13 02:08:16 ca Exp $
# Copyright (c) 2010 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# DKIM signing
# uses mcpmts.sh
# ----------------------------------------
#

# source directory of this (and other) shell script
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

# cd to right directory
cd ../qmgr || exit 1
rm -f ${MTMAP}

chkcto smtps MTA_USE_DKIM

if test X"`uname`" != "XOSF1"
then
while getopts O:p:s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# create directory to pass socket
test -d smtps || mkdir smtps

# create aliases
cat > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

# create mailertable (save old version?)
cat > ${MT} <<EOF
local.dom	lmtp:
example.net	[127.0.0.1]
EOF

Here=`pwd`
cd $SD/../libdkim || exit
K=`pwd`
cd $Here || exit

MSG=msg
OK=ok

# ----------------------------------------
# simple/simple
TEST=1
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
cat >${SMXCNF} <<EOF
smtps { log_level = 12;pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -9 ${DKTIME} -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
 DKIM_signing {
   sign {
    domain=example.net;
    selector=default;
    keyfile=${K}/private.key; }
 }
}
smtpc { log_level = 12;remote_port = ${SNKPORT}; wait_for_server=4;
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 12;wait_for_server=4; wait_for_client=4; 
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF}";
}
smar { log_level = 12;
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

clearlogs

cat > ${MSG} <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF

# ----------------------------------------
if ${SHELL} ${SD}/mcpmts.sh -13 lmtpsock -Ssender@example.net -Ruser@local.dom -E 0 -X 0 -A 1 -Z smtpc2 -c "-D ${MSG}" "$@"
then
  if test -s ${B1}
  then
     ${EGREP} -v '^Return-Path: ' ${B1} |\
	 ${SED} -e '/^Received:/,/^	id /d' > ${OUT}

     cat > ${OK} <<EOF
DKIM-Signature: v=1; a=rsa-sha256; c=simple/simple; d=example.net;
	s=default; t=1258487282;
	bh=P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E=; h=Subject:To:From;
	b=hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF
	 8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33Gt
	 HYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU=
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net

.
EOF
    if ${DIFF} -w ${OK} ${OUT} >/dev/null
    then
      :
    else
      ERR=true
    fi
  else
     ERR=true
  fi
else
  echo "$0: test ${TEST} failed"
  ERR=true
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# simple/relaxed
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat >${SMXCNF} <<EOF
smtps { log_level = 12;pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -9 ${DKTIME} -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
 DKIM_signing {
    body_canonicalization=relaxed;
   
   sign {
    domain=example.net;
    selector=default;
    keyfile=${K}/private.key; }
 }
}
smtpc { log_level = 12;remote_port = ${SNKPORT}; wait_for_server=4;
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 12;wait_for_server=4; wait_for_client=4; 
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF}";
}
smar { log_level = 12;
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

clearlogs

cat >${MSG} <<EOF
From: "This A. User" <user@example.com>
To: Not Really <recipient@sendmail.net>
Subject: test DKIM signing with MeTA1
Message-Id: <12345.abcde@example.net>
Date: Fri, 17 Dec 2010 17:20:05 -0800 (PST)

This is the message body.  It's nice, isn't it?
It  does not contain anything useful,  but that is the case with most e-mails. 

-- 
2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF

# ----------------------------------------
if ${SHELL} ${SD}/mcpmts.sh -13 lmtpsock -Ssender@example.net -Ruser@local.dom -E 0 -X 0 -A 1 -Z smtpc2 -c "-D ${MSG}" "$@"
then
  if test -s ${B1}
  then
     ${EGREP} -v '^Return-Path: ' ${B1} |\
	 ${SED} -e '/^Received:/,/^	id /d' > ${OUT}

     cat > ${OK} <<EOF
DKIM-Signature: v=1; a=rsa-sha256; c=simple/relaxed; d=example.net;
	s=default; t=1258487282;
	bh=3h44hVEPIImk1LStuM0LHplXSjdpK+7G0sB44W85dVk=; h=Date:Message-Id:Subject:To:From;
	b=LTzPv0wTq83vZY0jvfE1c4hiByoqirFbjmwZrl/uR9BD2cR/WIj7NCvlYqmK
	 wdm/br5OMsNg57H4AtTfv969CX/EAPJNHGyfn8nyDr3XYGa/ziXKVbfwrgXk7
	 JcICSKrwkcle3J32W1mVswbbdXcN12rdJSKdzPsIYZfJb6l4zE=
From: "This A. User" <user@example.com>
To: Not Really <recipient@sendmail.net>
Subject: test DKIM signing with MeTA1
Message-Id: <12345.abcde@example.net>
Date: Fri, 17 Dec 2010 17:20:05 -0800 (PST)

This is the message body.  It's nice, isn't it?
It  does not contain anything useful,  but that is the case with most e-mails. 

-- 
2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net

.
EOF

    if ${DIFF} -w ${OK} ${OUT} >/dev/null
    then
      :
    else
      ERR=true
    fi
  else
     ERR=true
  fi
else
  echo "$0: test ${TEST} failed"
  ERR=true
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# relaxed/simple
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
cat >${SMXCNF} <<EOF
smtps { log_level = 12;pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -9 ${DKTIME} -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
 DKIM_signing {
    header_canonicalization=relaxed;
   
   sign {
    domain=example.net;
    selector=default;
    keyfile=${K}/private.key; }
 }
}
smtpc { log_level = 12;remote_port = ${SNKPORT}; wait_for_server=4;
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
}
qmgr { log_level = 12;wait_for_server=4; wait_for_client=4; 
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF}";
}
smar { log_level = 12;
	start_action = wait; user = ${USER};
	restart_dependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

clearlogs

cat >${MSG} <<EOF
From : user@example.com
To:  recipient@sendmail.net,
	someone,
 else
Subject: test for
	DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF

# ----------------------------------------
if ${SHELL} ${SD}/mcpmts.sh -13 lmtpsock -Ssender@example.net -Ruser@local.dom -E 0 -X 0 -A 1 -Z smtpc2 -c "-D ${MSG}" "$@"
then
  if test -s ${B1}
  then
     ${EGREP} -v '^Return-Path: ' ${B1} |\
	 ${SED} -e '/^Received:/,/^	id /d' > ${OUT}

     cat > ${OK} <<EOF
DKIM-Signature: v=1; a=rsa-sha256; c=relaxed/simple; d=example.net;
	s=default; t=1258487282; 
	bh=P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E=; h=subject:to:from;
	b=Ac3I59vBl5ShKlewD6tRzLxAWlh/aXLxlAPB8j7EDT7AFL6Gm/nrhoOjK89+
	 tBPJKpLqOxFIGzm26pYMaPVleT2eslWiIooWt22HEJuF2lHOQWEaXXJ1fmQf/
	 BViMsCRQ2gfruFxXsdk2lhvObkgkz8LjCGbS/MxeBO1tuatBPo=
From : user@example.com
To:  recipient@sendmail.net,
	someone,
 else
Subject: test for
	DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net

.
EOF

    if ${DIFF} -w ${OK} ${OUT} >/dev/null
    then
      :
    else
      ERR=true
    fi
  else
     ERR=true
  fi
else
  echo "$0: test ${TEST} failed"
  ERR=true
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# # ----------------------------------------
# # simple/simple
# # large header: this header is currently too large, and we don't have
# # a good way to check for errors in this context, so disable it here
# # and use the libdkim tests instead.
# TEST=4
# if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
# then
# cat >${SMXCNF} <<EOF
# smtps { log_level = 12;pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
# 	listen_socket { type=inet; port=${SRVPORT}; }
# 	start_action = pass; user = ${USER};
# 	path = ../smtps/smtps;
# 	arguments = "smtps -9 ${DKTIME} -f ${SMXCNF} ${MTA_SERVER_OPTIONS}";
#  DKIM_signing {
#    sign {
#     domain=example.net;
#     selector=default;
#     keyfile=${K}/private.key; }
#  }
# }
# smtpc { log_level = 12;remote_port = ${SNKPORT}; wait_for_server=4;
# 	start_action = wait; user = ${USER};
# 	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF}";
# }
# qmgr { log_level = 12;wait_for_server=4; wait_for_client=4; 
# 	start_action = wait; user = ${USER};
# 	restart_dependencies = { smtps,smtpc,smar, };
# 	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF}";
# }
# smar { log_level = 12;
# 	start_action = wait; user = ${USER};
# 	restart_dependencies = { smtps,qmgr };
# 	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
# }
# EOF
# 
# clearlogs
# 
# cat > ${MSG} <<EOF
# From: user@example.com
# To: recipient@sendmail.net
# Subject: test DKIM
# HEADERNAME: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME1: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME2: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME3: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME4: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME5: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME6: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME7: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME8: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME9: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME10: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME11: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME12: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME13: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME14: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME15: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME16: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME17: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME18: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME19: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME20: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME21: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME22: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME23: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME24: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# 
# 2020 Vision Street
# Anycity, Anystate 90009
# Office ph: (510) 555-1234
# Cell ph:   (510) 555-5678
# fax:       (510) 555-9000
# email: mmtest3@v253.pogonip.net
# EOF
# 
# # ----------------------------------------
# if ${SHELL} ${SD}/mcpmts.sh -13 lmtpsock -Ssender@example.net -Ruser@local.dom -E 0 -X 0 -A 1 -Z smtpc2 -c "-D ${MSG}" "$@"
# then
#   if test -s ${B1}
#   then
#      ${EGREP} -v '^Return-Path: ' ${B1} |\
# 	 ${SED} -e '/^Received:/,/^	id /d' > ${OUT}
# 
#      cat > ${OK} <<EOF
# DKIM-Signature: v=1; a=rsa-sha256; c=simple/simple; d=example.net;
# 	s=default; t=1258487282;
# 	bh=P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E=; h=Subject:To:From;
# 	b=hU1eiDdhQCwLZh1jXoUNF9Sg++8hJfXb4823xLTxNK2y6nMc6M6/iRudoNbF
# 	 8/ijn/by/PnVE94Jphmzbr9OCfwh0zy+L9wd8ajH7ge7zPjQgUvuiLIzf33Gt
# 	 HYSrSGG4ry+LkUJNcKvjxI7UDSRlnodX6X3V6dWpto6l06yNeU=
# From: user@example.com
# To: recipient@sendmail.net
# Subject: test DKIM
# HEADERNAME: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME1: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME2: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME3: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME4: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME5: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME6: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME7: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME8: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME9: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME10: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME11: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME12: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME13: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME14: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME15: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME16: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME17: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME18: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME19: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME20: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME21: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME22: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME23: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME24: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
# 
# text.
# 
# 
# 2020 Vision Street
# Anycity, Anystate 90009
# Office ph: (510) 555-1234
# Cell ph:   (510) 555-5678
# fax:       (510) 555-9000
# email: mmtest3@v253.pogonip.net
# 
# .
# EOF
#     if ${DIFF} -w ${OK} ${OUT} >/dev/null
#     then
#       :
#     else
#       ERR=true
#     fi
#   else
#      ERR=true
#   fi
# else
#   echo "$0: test ${TEST} failed"
#   ERR=true
#   test X"${MTA_STOPONERROR}" != X && exit 1
# fi
# if ${ERR}
# then
#     echo "$0: test ${TEST} check failed"
#     ERRS=`expr ${ERRS} + 1 `
#     test X"${MTA_STOPONERROR}" != X && exit 1
# else
#     ${VERBOSE} && echo "test ${TEST} succeeded"
# fi
# else
#   ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
# fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
#  rm -f ${MSG} ${OK} ${OUT}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
