#!/bin/sh
# $Id: t-pm-regex-0.sh,v 1.29 2013/10/19 19:35:37 ca Exp $
# Copyright (c) 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test milter-regex
# uses smtpcnf.sh.
# ----------------------------------------
#

if test "X"${MTA_PMILTER_REGEX_TEST} = "X"
then
  echo "SKIPPED: $0: MTA_PMILTER_REGEX_TEST is not set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps MTA_USE_PMILTER '(--enable-pmilter)'

PMPRG="../contrib/milter-regex"
if test -x ${PMPRG}
then
  :
else
  echo "SKIPPED: $0: ${PMPRG} missing"
  exit 0
fi

PMCONF="regex.conf"
PMOPT="-d -c ${PMCONF} -p pmilter.sock"
MSG="t-pm-regex-0.msg"

if test X"`uname`" != "XOSF1"
then
while getopts O:s:V FLAG
do
  case "${FLAG}" in
    O) PMOPT="${PMOPT} ${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || myexit 1
rm -f ${MTMAP}

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 12;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
policy_milter { socket {type=unix; path="pmilter.sock";} timeout=2; }
}
EOF

# ----------------------------------------
# reject Subject in milter
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
reject "Subject matches word and this rejection message is too long to be printed in full"
header /Subject/ /${WB}word${WE}/e
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: this is a word that should be rejected

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
  if ${GREP} 'From=sender@local.domain, To=rcpt@local.domain' ${PML} >/dev/null
  then
    :
  else
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# accept Subject in milter
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: words

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi


# ----------------------------------------
# reject Subject in milter
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
reject "Subject matches non-ascii"
header /Subject/ /^([[:print:]]|[[:cntrl:]])*$/ne
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: 0-=\\!@#\$%^&*()_+|t~[];',./<>?:"{}abc

0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -2 -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
    :
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# accept Subject in milter
TEST=4
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
reject "Subject matches non-ascii"
header /Subject/ /^([[:print:]]|[[:cntrl:]])*$/ne
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: 0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
	continue here

0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi


# ----------------------------------------
# reject Subject in milter
TEST=5
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
reject "Subject matches non-ascii"
header /Subject/ /^([[:print:]]|[[:cntrl:]])*$/ne
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: first line ok but then
 0-=\\!@#\$%^&*()_+|t~[];',./<>?:"{}abc

0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
    :
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi


# ----------------------------------------
# Subject would be rejected but recipient is whitelisted
TEST=6
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
accept "non-filtered address/
envrcpt /@example\.org/n

reject "Subject matches non-ascii"
header /Subject/ /^([[:print:]]|[[:cntrl:]])*$/ne
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: 0-=\\!@#\$%^&*()_+|t~[];',./<>?:"{}abc

0-=\\!@#\$%^&*()_+|~[];',./<>?:"{}abc
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# discard due to Subject match in milter
TEST=7
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
discard
header /Subject/ /${WB}dirty word${WE}/
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: here is a dirty word

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 1 -X 0 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject Subject in milter
TEST=8
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
reject "Subject matches word and this rejection message is too long to be printed in full"
header /Subject/ /${WB}word${WE}/e
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: word-begin that should be rejected

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
    :
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject envrcpt in milter but only if Subject exists
# this allows us to log the Subject too.
TEST=9
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
tempfail "where did you get this address?"
envrcpt /x@y\.z/ and header /Subject/ /./
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: this is logged

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -1 -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
  if ${GREP} 'From=sender@local.domain, To=rcpt@local.domain' ${PML} >/dev/null
  then
    :
  else
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject envfrom in milter but only if connection from a certain host
TEST=10
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
tempfail "Yahoo does not have a working abuse address -- get a real e-mail provider"
envfrom /@yahoo\.com/ and header /Subject/ /./ and not connect /example\.com/ //
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: subject ${TEST}

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -1 -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"a@yahoo.com" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
  if ${GREP} 'TEMPFAIL=Yahoo does not have a working abuse address' ${PML} >/dev/null
  then
    :
  else
    ERR=true
    echo "$0: test ${TEST} check failed: missing TEMPFAIL=Yahoo"
  fi
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject envfrom in milter but only if connection from a certain host
TEST=11
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
getip2 2
${CAT} > ${PMCONF} <<EOF
tempfail "Yahoo does not have a working abuse address -- get a real e-mail provider"
envfrom /@yahoo\.com/ and not connect // /$H2/
EOF
${CAT} > ${MSG} <<EOF
From: sender@local.domain
To: rcpt@local.domain
Message-Id: <t-${TEST}-123456789@local.dom>
Subject: subject ${TEST}

t-${TEST}-123456789@local.dom
EOF
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -1 -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG} -b $H2" -E 1 -A 1 -X 1 -S"a@yahoo.com" -R"x@y.z" -V "-C $H2" "$@"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi


# ----------------------------------------
# reject List-Unsubscribe in milter
TEST=12
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${PMCONF} <<EOF
reject "not subscribed"
header /^List-Unsubscribe/ /./ and header /^Received/ /by services\.sendmail\.com/ and header /^Subject/ /./

tempfail "most likely spam - 3"
envfrom /@[a-z0-9-]*\.[a-z0-9-]*\.[a-z0-9-]*/i and header /^Subject/ /./ and header /^Received/ /by services\.sendmail\.com/ and not header /^List-Unsubscribe/ /./
EOF
${CAT} > ${MSG} <<EOF
Received: from smtp6s.sent01.sentyou.net (smtp6s.sent01.sentyou.net [188.165.123.156])
	by services.sendmail.com (8.14.5/8.14.6) with ESMTP id r49Ge4fn015214
	for <luser@sendmail.org>; Thu, 9 May 2013 09:40:07 -0700 (PDT)
	(envelope-from aikade@twoday.com.br)
To: luser@sendmail.org
Subject: Ofertas Aikade jogos Ps3 e Xbox360 Originais por R$69,90
Message-ID: <f27656d091936b6f7142bfe52b103161@sentyou.net>
Date: Thu, 09 May 2013 13:29:53 -0300
From: "Aikade" <aikade@twoday.com.br>
Reply-To: aikade@twoday.com.br
MIME-Version: 1.0
X-Mailer-LID: 129
List-Unsubscribe: <http://sentyou.net/envio/unsubscribe.php?M=11725390&C=5d2059ae54847c2d3e32beea268d58ba&L=129&N=385>
X-Mailer-RecptId: 11725390
X-Mailer-SID: 385
X-Mailer-Sent-By: 3

t-${TEST}-123456789@local.dom
EOF
# twoday.com.br does not resolve, breaks tests, use "good" sender domain
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -p ${PMPRG} -P "${PMOPT}" -Z 'smtpc2' -c "-E -D ${MSG}" -E 0 -A 0 -X 0 "$@" -S"aikade@meta1.org" -R"ca+sendmail.org@esmtp.org"
then
    ERR=false
else
    echo "$0: test ${TEST} failed."
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error, text=5.* not subscribed' ${CLTL} >/dev/null
then
    :
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${MSG}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  myexit 1
fi
