#!/bin/sh
# $Id: t-tls-calist-0.sh,v 1.3 2013/10/20 20:03:44 ca Exp $
#
# Copyright (c) 2012 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test TLS: CAlist
# ----------------------------------------
#

SD=`dirname $0`
THISDIR=`pwd`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps MTA_USE_TLS
chkcto smtpc MTA_USE_TLS

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to working directory for alias file!
cd ../qmgr || exit 1

setuptls0

# ----------------------------------------
# test: calist set to ""
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
tls { calist_file=""; }
Log_Level = 12;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender2@other.dom -Ruser@host.local -E 1 -X 1 -A 1 -q
then
  if fgrep 'No client certificate CA names sent' ${CLTL} >/dev/null
  then
    :
  else
    echo "$0: ${TEST}: missing No client certificate CA names sent in ${CLTL}"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: calist not set
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 12;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender2@other.dom -Ruser@host.local -E 1 -X 1 -A 1 -q
then
  if grep '^Acceptable client certificate CA names' ${CLTL} >/dev/null &&
     grep '^/C=US/ST=California/L=Berkeley/O=Example Org/OU=MTA/CN=CA/emailAddress=CA@example\.org$' ${CLTL} >/dev/null
  then
    :
  else
    echo "$0: ${TEST}: missing client certificate CA names in ${CLTL}"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# test: calist set to different CA
TEST=3
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
tls { calist_file=C2cert.pem; }
Log_Level = 12;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
}
EOF

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -Ssender2@other.dom -Ruser@host.local -E 1 -X 1 -A 1 -q
then
  if grep '^Acceptable client certificate CA names' ${CLTL} >/dev/null &&
     grep '^/C=US/ST=California/L=Berkeley/O=Example Com/OU=MTA/CN=CA/emailAddress=CA@example\.com$' ${CLTL} >/dev/null
  then
    :
  else
    echo "$0: ${TEST}: missing client certificate CA names in ${CLTL}"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

testend
