#!/bin/sh
# $Id: t-tls-reqr-9.sh,v 1.1 2013/11/03 16:02:48 ca Exp $
#
# Copyright (c) 2013 Claus Assmann
#
# By using this file, you agree to the terms and conditions set
# forth in the license/LICENSE.3C file which can be found at the
# top level of this source code distribution.
#
# ----------------------------------------
# test TLS requirements in smtpc; use chain of CAs
# creates two MTA (one in chkmts, one in qmgr) to use STARTTLS
# run smtpcnf.sh with different parameters
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

tlsrintro "$@"
setuptlsr0 "" "" \
 "smtpc_rcpt_conf:@local.dom	tls_requirements { flags={verified}; cert_fps = { E7:02:5D:4E:7B:B4:DC:CE:38:C1:A4:11:50:A0:18:9D:D1:20:44:8A }; }" \
 "tls { cert_file=cert4.pem; key_file=key4.pem; CAcert_file = ${CAS}; }"

# ----------------------------------------
# test: send mail to sink, requirements fulfilled
TEST=1
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -3 lmtpsock -Ssender${TEST}@other.dom -Ruser@local.dom -E 0 -X 0 -A 1
then
  chkTLS ${B1}
  cd ${THISDIR}
  egrep 'ERROR|FATAL|ALERT' ${SCL1} && ERR=true
  cd ../qmgr
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# test: send mail to sink, requirements not fulfilled
TEST=2
if echo " ${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

cd ${THISDIR}
${CAT} > ${ACC} <<EOF
smtpc_rcpt_conf:@local.dom	tls_requirements { flags={verified}; cert_fps = { E7:02:5D:4E:7B:B4:DC:CE:38:C1:A4:11:50:A0:18:9D:D1:20:44:8B } }
EOF
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} < ${ACC}
# notify smar
kill -USR1 ${SMARPID}
cd ../qmgr

if ${SHELL} ${SD}/smtpcnf.sh -F ${SRVPORT} -3 lmtpsock -Ssender${TEST}@other.dom -Ruser@local.dom -E 0 -X 0 -A 1
then
  if test -s ${B1}
  then
    ${VERBOSE} && echo "$0: test ${TEST}: found ${B1}"
    ERR=true
  else
    # check that mail is in queue, check error message...
    # how?
    cd ${THISDIR} || ERR=true
    ls ?/S* >/dev/null || ERR=true
    egrep 'ERROR|FATAL|ALERT' ${SCL1} && ERR=true
    cd ../qmgr
  fi
else
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
fi
rm -f ${B1}
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


testend rmb
