/*
 * Copyright (c) 2002-2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbputrec.c,v 1.27 2011/01/30 21:49:05 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/varargs.h"
#include "sm/strrcb.h"
#include "sm/rcb.h"
#include "sm/reccom.h"
#include "sm/str-int.h"
#if !SM_NO_CSTR
# include "sm/cstr.h"
#endif
#include "sm/net.h"

/*
**  SM_RCB_PUTREC -- Encode an entire record into an sm_rcb_P.
**
**	Parameters:
**		rcb -- sm_rcb_P object to append onto.
**		flags -- flags
**		sz -- size of record
**		n -- size of for sm_rcb_open_enc(); see comments there!
**		... -- data
**
**	Returns:
**		usual sm_error code; ENOMEM, SM_E_OVFLW_SC, SM_E_OVFLW_NS,
**			EINVAL, etc?
*/

sm_ret_T
sm_rcb_putrec(sm_rcb_P rcb, uint flags, uint32_t sz, int n, ...)
{
	int k;
	uint32_t rt, v;
	sm_str_P str;
#if !SM_NO_CSTR
	sm_cstr_P cstr;
#endif
	uchar *buf;
	uint len;
	sm_ret_T ret;
	sm_sockaddr_P sockaddr;
	sm_ipaddr_P ipaddr;
	va_list ap;

	SM_IS_RCB(rcb);
	if (SM_IS_FLAG(flags, RCB_PUTR_OPEN)) {
		ret = sm_rcb_open_enc(rcb, n);
		if (sm_is_err(ret))
			goto error;
	}

	if (SM_IS_FLAG(flags, RCB_PUTR_FIRST)) {
		/* encode rcb: placeholder for size */
		ret = sm_rcb_putuint32(rcb, sz);
		if (sm_is_err(ret))
			goto err2;
	}

/* for code in rcbputm.h */
#if MTA_USE_RCBV_STRN
# undef MTA_USE_RCBV_STRN
# define MTA_USE_RCBV_STRN 0
#endif

	va_start(ap, n);
#include "rcbputm.h"
	va_end(ap);
	if (SM_IS_FLAG(flags, RCB_PUTR_CLOSE))
		ret = sm_rcb_close_enc(rcb);

	return SM_SUCCESS;

  err1:
	va_end(ap);
  err2:
	if (SM_IS_FLAG(flags, RCB_PUTR_CLOSE))
		(void) sm_rcb_close_enc(rcb);
  error:
	return ret;
}
