/*
 * Copyright (c) 2011 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: smcert.c,v 1.5 2014/04/19 15:55:30 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/io.h"
#include "sm/tls.h"

#if MTA_USE_TLS

/*
**  SM_TLS_GET_CN -- extract CN from an X509 name
**
**	Parameters:
**		x509_name -- X509 name
**		nid -- index
**		str -- where to write result (output)
**
**	Returns:
**		usual error code
*/

sm_ret_T
sm_tls_get_cn(X509_NAME *x509_name, int nid, sm_str_P *str)
{
	int r;
	char buf[SM_X509_NAME_MAX];

#define CHECK_X509_NAME	do {	\
		if (r == -1)	\
			(void) strlcpy(buf, "BadCert:Unknown", sizeof(buf));	\
		else if ((size_t)r >= sizeof(buf) - 1)	\
			(void) strlcpy(buf, "BadCert:TooLong", sizeof(buf));	\
		else if ((size_t)r > strlen(buf))	\
			(void) strlcpy(buf, "BadCert:ContainsNUL", sizeof(buf));	\
	} while (0)

	r = X509_NAME_get_text_by_NID(x509_name, nid, buf, sizeof buf);
	CHECK_X509_NAME;
	TLS_SET_XSTR(buf, *str);
	return SM_SUCCESS;
}
#endif /* MTA_USE_TLS */
