#!/bin/sh
# $Id: t-async-1.sh,v 1.3 2013/09/22 05:11:45 ca Exp $
#
# ------------------------------------------------------------
# Description:
# test new implementation of async lookups: reverse/forward lookup
# ------------------------------------------------------------
#

if test X"${MTA_NO_DNS_TEST}" != "X"
then
  echo "SKIPPED: $0: MTA_NO_DNS_TEST is set"
  exit 0
fi
# this test requires DNS entries for 10.0.255.*
if test X"${MTA_HAVE_10_INADDRARPA}" != "Xtrue"
then
  echo "SKIPPED: $0: MTA_HAVE_10_INADDRARPA is not set to true"
  exit 0
fi

TPRG=./t-async-0
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../smar
fi
. ${SD}/common.sh

CONF=smar.conf
RMTMP=true

${TPRG} -h 2>&1 | grep SKIP && exit 0

if test X"`uname`" != "XOSF1"
then
while getopts kV FLAG
do
  case "${FLAG}" in
    k) RMTMP=false;;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

rm -f ${ACCMAP} ${SOCK} ${PIDS}

# create aliases map
[ -s ${ALIMAP} ] || ${MM} </dev/null

cat > ${CONF} <<EOF
smar {
  Log_Level = 12;
  DNS_timeout = 5;
  dnsbl { domain = dnsbl.meta1.org.; tag = "meta1dnsbl"; }
  dnsbl { domain = bl.meta1.org.; tag = "meta1bl"; }
  dnsbl { domain = all.meta1.org.; tag = "meta1all"; }
}
EOF

# create access map for tests
${MM} -t'	' -F ${ACCMAP} <<EOF
cltaddr:10.0.255.9	ok
cltresolve:NO	error:450 4.7.1 get a name
cltname:one.meta1.org	error:451 4.7.2 one.meta1.org
EOF

chk()
{
if grep "$1" ${OUT} >/dev/null
then
  :
else
  $VERBOSE && echo "$0: missing $1 in ${OUT}"
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed."
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
}

TEST=1
# ----------------------------------------
# match
${VALGRIND} ${TPRG} -i ${NS} -r 10.0.255.1 > ${OUT} 2>&1
chk '^func=t_smar_res, status=399,'
chk '^func=t_smar_cltname, hostname=one.meta1.org.'

TEST=2
# ----------------------------------------
# does not match
${VALGRIND} ${TPRG} -i ${NS} -r 10.0.255.2 > ${OUT} 2>&1
chk '^func=t_smar_res, status=450,'

TEST=3
# ----------------------------------------
# forward lookup fails
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.3 > ${OUT} 2>&1
chk '^func=t_smar_rvrs_notify, rescnt=0, ret=8,'

TEST=4
# ----------------------------------------
# reverse lookup fails
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.10 > ${OUT} 2>&1
chk '^func=t_smar_rvrs_notify, rescnt=0, ret=4,'

# ; two hostnames map to one PTR record
# 2host1ptr1.meta1.org.	IN	A	10.0.255.21
# 2host1ptr2.meta1.org.	IN	A	10.0.255.21
# ; one hostname maps to two PTR record
# 1host2ptr.meta1.org.	IN	A	10.0.255.120
# 1host2ptr.meta1.org.	IN	A	10.0.255.121

TEST=5
# ----------------------------------------
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.21 > ${OUT} 2>&1
chk '^func=t_smar_cltname, hostname=2host1ptr[12]\.meta1\.org\.'

TEST=6
# ----------------------------------------
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.120 > ${OUT} 2>&1
chk '^func=t_smar_cltname, hostname=1host2ptr\.meta1\.org\.'

TEST=7
# ----------------------------------------
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.121 > ${OUT} 2>&1
chk '^func=t_smar_cltname, hostname=1host2ptr\.meta1\.org\.'

TEST=8
# ----------------------------------------
# does not match, but overridden by cltaddr:
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.9 > ${OUT} 2>&1
chk '^func=t_smar_res, status=0,'
chk 'func=t_smar_rvrs_notify, rescnt=0, ret=2,'

TEST=9
# ----------------------------------------
# does not match
${VALGRIND} ${TPRG} -i ${NS} -d 9 -R 10.0.255.4 > ${OUT} 2>&1
chk '^func=t_smar_res, status=450,'
chk 'func=t_smar_rvrs_notify, rescnt=0, ret=2,'

if test ${ERRS} -eq 0
then
  ${RMTMP} && rm -f ${OUT} ${CONF}
fi

exit ${ERRS}
